@@ -37,13 +37,15 @@ from typing import Optional, Union
 
 from osgeo import gdal, ogr
 from osgeo_utils.auxiliary.gdal_argparse import GDALArgumentParser, GDALScript
-from osgeo_utils.auxiliary.util import GetOutputDriverFor
+from osgeo_utils.auxiliary.util import GetOutputDriverFor, enable_gdal_exceptions
 
 
+@enable_gdal_exceptions
 def gdal_polygonize(
     src_filename: Optional[str] = None,
     band_number: Union[int, str] = 1,
     dst_filename: Optional[str] = None,
+    overwrite: bool = False,
     driver_name: Optional[str] = None,
     dst_layername: Optional[str] = None,
     dst_fieldname: Optional[str] = None,
@@ -106,7 +108,36 @@ def gdal_polygonize(
         dst_ds = ogr.Open(dst_filename, update=1)
         gdal.PopErrorHandler()
     except Exception:
-        dst_ds = None
+        try:
+            dst_ds = ogr.Open(dst_filename)
+        except Exception:
+            dst_ds = None
+        if dst_ds and not overwrite:
+            raise Exception(
+                f"{dst_filename} exists, but cannot be updated. You may need to remove it before or use -overwrite"
+            )
+
+    if dst_ds is not None and overwrite:
+        cnt = dst_ds.GetLayerCount()
+        iLayer = None  # initialize in case there are no loop iterations
+        for iLayer in range(cnt):
+            poLayer = dst_ds.GetLayer(iLayer)
+            if poLayer is not None and poLayer.GetName() == dst_layername:
+                break
+
+        delete_ok = False
+        if iLayer != cnt:
+            if dst_ds.TestCapability(ogr.ODsCDeleteLayer) == 1:
+                try:
+                    delete_ok = dst_ds.DeleteLayer(iLayer) == ogr.OGRERR_NONE
+                except Exception:
+                    delete_ok = False
+
+        if not delete_ok:
+            if cnt == 1:
+                dst_ds = None
+                if gdal.VSIStatL(dst_filename):
+                    gdal.Unlink(dst_filename)
 
     # =============================================================================
     # 	Create output file.
@@ -116,6 +147,9 @@ def gdal_polygonize(
         if not quiet:
             print("Creating output %s of format %s." % (dst_filename, driver_name))
         dst_ds = drv.CreateDataSource(dst_filename)
+        if dst_ds is None:
+            print('Cannot create datasource "%s"' % dst_filename)
+            return 1
 
     # =============================================================================
     #       Find or create destination layer.
@@ -197,7 +231,7 @@ class GDALPolygonize(GDALScript):
             the pixel value of that polygon.
             A raster mask may also be provided to determine which pixels are eligible for processing.
             The utility will create the output vector datasource if it does not already exist,
-            defaulting to GML format.
+            otherwise it will try to append to an existing one.
             The utility is based on the GDALPolygonize() function
             which has additional details on the algorithm."""
         )
@@ -282,6 +316,13 @@ class GDALPolygonize(GDALScript):
             help="Specify a layer creation option. This may be specified multiple times.",
         )
 
+        parser.add_argument(
+            "-overwrite",
+            dest="overwrite",
+            action="store_true",
+            help="overwrite output file if it already exists",
+        )
+
         parser.add_argument(
             "src_filename",
             type=str,
